import 'package:flutter/material.dart';
import 'package:osam/osam.dart';

part 'main.g.dart';

@HiveType(typeId: 0)
class Counter {
  @HiveField(0)
  final IProperty<int> value;

  Counter({IProperty<int> value}) : value = value ?? IProperty(0);
}

class UseCase2 extends UseCase {
  final Counter counter;

  UseCase2(this.counter);

  void increment() => setValue(counter.value, counter.value.value + 1);
}

final persist = Persist<Counter, Null>(appName: '123');

void main() async {
  WidgetsFlutterBinding.ensureInitialized();
  Hive.registerAdapter(CounterAdapter());
  registerOsamAdapters();
  await persist.init();
  final counter = persist.appState ?? Counter();
  final usecaseContainer = UseCaseContainer([UseCase2(counter)]);

  runApp(Example(
    counter: counter,
    useCaseContainer: usecaseContainer,
  ));
}

class Example extends StatelessWidget {
  final Counter counter;
  final UseCaseContainer useCaseContainer;

  const Example({Key key, this.counter, this.useCaseContainer})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      home: Scaffold(
        floatingActionButton: FloatingActionButton(
          child: Icon(Icons.add),
          onPressed: () {
            useCaseContainer.useCase<UseCase2>().increment();
            persist.save(state: counter, uiState: null, version: null);
          },
        ),
        body: Center(
          child: ValueListenableBuilder(
            valueListenable: counter.value,
            builder: (_, value, __) => Text(
              value.toString(),
              style: const TextStyle(fontSize: 40),
            ),
          ),
        ),
      ),
    );
  }
}
