import '../../osam.dart';
import 'package:equatable/equatable.dart';
import 'package:flutter/foundation.dart';
import 'package:osam/osam.dart';

part '../domain/property_notifier.dart';

part 'usecase_container.g.dart';

class UseCaseNotFoundException implements Exception {
  final Type useCaseType;

  UseCaseNotFoundException(this.useCaseType);
}

class UseCaseContainer {
  final List<UseCase> _useCases;
  final _useCasesMap = <Type, UseCase>{};

  U useCase<U extends UseCase>() {
    final useCase = _useCasesMap[U];
    if (useCase == null) {
      throw UseCaseNotFoundException(U);
    } else
      return useCase;
  }

  UseCaseContainer(this._useCases) {
    if (_useCases?.isNotEmpty ?? false) {
      for (final useCase in _useCases) {
        useCase._useCaseContainer = this;
        _useCasesMap[useCase.runtimeType] = useCase;
      }
    }
  }

  void updateUseCases(List<Object> newUseCases) => newUseCases
      .forEach((useCase) => _useCasesMap[useCase.runtimeType] = useCase);
}

class UseCase {
  UseCaseContainer _useCaseContainer;

  UseCaseContainer get useCaseContainer => _useCaseContainer;

  void setValue<T>(IProperty<T> notifier, T value) =>
      (notifier as _PropertyNotifier).value = value;

  void updateValue<T>(IProperty<T> notifier, Update<T> update) =>
      (notifier as _PropertyNotifier).update(update);
}

mixin PropertyChanger {
  void setValue<T>(IProperty<T> notifier, T value) =>
      (notifier as _PropertyNotifier).value = value;
  void updateValue<T>(IProperty<T> notifier, Update<T> update) =>
      (notifier as _PropertyNotifier).update(update);
}
